package xhttp3.internal.http;

import java.io.IOException;
import java.io.InterruptedIOException;
import java.net.HttpRetryException;
import java.net.ProtocolException;
import java.net.Proxy;
import java.net.ProxySelector;
import java.net.SocketTimeoutException;
import java.net.Proxy.Type;
import java.security.cert.CertificateException;
import java.util.List;
import javax.net.SocketFactory;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLHandshakeException;
import javax.net.ssl.SSLPeerUnverifiedException;
import javax.net.ssl.SSLSocketFactory;
import xhttp3.Address;
import xhttp3.Authenticator;
import xhttp3.CertificatePinner;
import xhttp3.Connection;
import xhttp3.ConnectionPool;
import xhttp3.Dns;
import xhttp3.HttpUrl;
import xhttp3.Interceptor;
import xhttp3.OkHttpClient;
import xhttp3.Request;
import xhttp3.RequestBody;
import xhttp3.Response;
import xhttp3.ResponseBody;
import xhttp3.Route;
import xhttp3.internal.Util;
import xhttp3.internal.connection.RealConnection;
import xhttp3.internal.connection.RouteException;
import xhttp3.internal.connection.StreamAllocation;

public final class RetryAndFollowUpInterceptor implements Interceptor {
   private static final int MAX_FOLLOW_UPS = 20;
   private final OkHttpClient client;
   private StreamAllocation streamAllocation;
   private boolean forWebSocket;
   private volatile boolean canceled;

   public RetryAndFollowUpInterceptor(OkHttpClient client) {
      this.client = client;
   }

   private Address createAddress(HttpUrl url) {
      SSLSocketFactory var2 = null;
      HostnameVerifier var3 = null;
      CertificatePinner var4 = null;
      if (url.isHttps()) {
         var2 = this.client.sslSocketFactory();
         var3 = this.client.hostnameVerifier();
         var4 = this.client.certificatePinner();
      }

      HttpUrl var10002 = url;
      String url1 = url.host();
      int var5 = var10002.port();
      String var14 = url1;
      RetryAndFollowUpInterceptor var10003 = this;
      RetryAndFollowUpInterceptor var10004 = this;
      RetryAndFollowUpInterceptor var10005 = this;
      RetryAndFollowUpInterceptor var10006 = this;
      RetryAndFollowUpInterceptor var10007 = this;
      RetryAndFollowUpInterceptor var10008 = this;
      Dns var11 = this.client.dns();
      SocketFactory url2 = var10008.client.socketFactory();
      Authenticator var6 = var10007.client.proxyAuthenticator();
      Proxy var7 = var10006.client.proxy();
      List var8 = var10005.client.protocols();
      List var9 = var10004.client.connectionSpecs();
      ProxySelector var10 = var10003.client.proxySelector();
      return new Address(var14, var5, var11, url2, var2, var3, var4, var6, var7, var8, var9, var10);
   }

   private boolean recover(IOException e, boolean routeException, Request userRequest) {
      this.streamAllocation.streamFailed(e);
      if (!this.client.retryOnConnectionFailure()) {
         return false;
      } else if (!routeException && userRequest.body() instanceof UnrepeatableRequestBody) {
         return false;
      } else if (!this.isRecoverable(e, routeException)) {
         return false;
      } else {
         return this.streamAllocation.hasMoreRoutes();
      }
   }

   private boolean isRecoverable(IOException e, boolean routeException) {
      if (e instanceof ProtocolException) {
         return false;
      } else if (!(e instanceof InterruptedIOException)) {
         if (e instanceof SSLHandshakeException && e.getCause() instanceof CertificateException) {
            return false;
         } else {
            return !(e instanceof SSLPeerUnverifiedException);
         }
      } else {
         return e instanceof SocketTimeoutException && routeException;
      }
   }

   private Request followUpRequest(Response userResponse) {
      if (userResponse != null) {
         RealConnection var2;
         Route var5;
         if ((var2 = this.streamAllocation.connection()) != null) {
            var5 = var2.route();
         } else {
            var5 = null;
         }

         int var10000 = userResponse.code();
         String var3 = userResponse.request().method();
         switch (var10000) {
            case 307:
            case 308:
               if (!var3.equals("GET") && !var3.equals("HEAD")) {
                  return null;
               }
            case 300:
            case 301:
            case 302:
            case 303:
               if (!this.client.followRedirects()) {
                  return null;
               } else {
                  String var6;
                  if ((var6 = userResponse.header("Location")) == null) {
                     return null;
                  } else {
                     HttpUrl var7;
                     if ((var7 = userResponse.request().url().resolve(var6)) == null) {
                        return null;
                     } else {
                        if (!var7.scheme().equals(userResponse.request().url().scheme()) && !this.client.followSslRedirects()) {
                           return null;
                        }

                        Request.Builder var4 = userResponse.request().newBuilder();
                        if (HttpMethod.permitsRequestBody(var3)) {
                           if (HttpMethod.redirectsToGet(var3)) {
                              var4.method("GET", (RequestBody)null);
                           } else {
                              var4.method(var3, (RequestBody)null);
                           }

                           var4.removeHeader("Transfer-Encoding");
                           var4.removeHeader("Content-Length");
                           var4.removeHeader("Content-Type");
                        }

                        if (!this.sameConnection(userResponse, var7)) {
                           var4.removeHeader("Authorization");
                        }

                        return var4.url(var7).build();
                     }
                  }
               }
            case 401:
               return this.client.authenticator().authenticate(var5, userResponse);
            case 407:
               if ((var5 != null ? var5.proxy() : this.client.proxy()).type() == Type.HTTP) {
                  return this.client.proxyAuthenticator().authenticate(var5, userResponse);
               }

               throw new ProtocolException("Received HTTP_PROXY_AUTH (407) code while not using proxy");
            case 408:
               if (userResponse.request().body() instanceof UnrepeatableRequestBody) {
                  return null;
               }

               return userResponse.request();
            default:
               return null;
         }
      } else {
         throw new IllegalStateException();
      }
   }

   private boolean sameConnection(Response response, HttpUrl followUp) {
      HttpUrl var3;
      return (var3 = response.request().url()).host().equals(followUp.host()) && var3.port() == followUp.port() && var3.scheme().equals(followUp.scheme());
   }

   public void cancel() {
      this.canceled = true;
      StreamAllocation var1;
      if ((var1 = this.streamAllocation) != null) {
         var1.cancel();
      }

   }

   public boolean isCanceled() {
      return this.canceled;
   }

   public OkHttpClient client() {
      return this.client;
   }

   public void setForWebSocket(boolean forWebSocket) {
      this.forWebSocket = forWebSocket;
   }

   public boolean isForWebSocket() {
      return this.forWebSocket;
   }

   public StreamAllocation streamAllocation() {
      return this.streamAllocation;
   }

   public Response intercept(Interceptor.Chain chain) {
      Request var2 = chain.request();
      ConnectionPool var3 = this.client.connectionPool();
      this.streamAllocation = new StreamAllocation(var3, this.createAddress(var2.url()));
      int var50 = 0;
      Response var4 = null;

      while(!this.canceled) {
         Response var55;
         Response var56;
         label500: {
            Throwable var10000;
            label508: {
               IOException var5;
               boolean var10001;
               boolean var54;
               label509: {
                  RouteException var52;
                  try {
                     try {
                        var55 = var4;
                        var56 = ((RealInterceptorChain)chain).proceed(var2, this.streamAllocation, (HttpStream)null, (Connection)null);
                        break label500;
                     } catch (RouteException var45) {
                        var52 = var45;
                     } catch (IOException var46) {
                        var5 = var46;
                        break label509;
                     }
                  } catch (Throwable var47) {
                     var10000 = var47;
                     var10001 = false;
                     break label508;
                  }

                  try {
                     var54 = this.recover(var52.getLastConnectException(), true, var2);
                  } catch (Throwable var44) {
                     var10000 = var44;
                     var10001 = false;
                     break label508;
                  }

                  if (var54) {
                     continue;
                  }

                  try {
                     throw var52.getLastConnectException();
                  } catch (Throwable var43) {
                     var10000 = var43;
                     var10001 = false;
                     break label508;
                  }
               }

               try {
                  var54 = this.recover(var5, false, var2);
               } catch (Throwable var42) {
                  var10000 = var42;
                  var10001 = false;
                  break label508;
               }

               if (var54) {
                  continue;
               }

               label482:
               try {
                  throw var5;
               } catch (Throwable var41) {
                  var10000 = var41;
                  var10001 = false;
                  break label482;
               }
            }

            Throwable chain1 = var10000;
            this.streamAllocation.streamFailed((IOException)null);
            this.streamAllocation.release();
            throw chain1;
         }

         Response var49 = var56;
         if (var55 != null) {
            var49 = var49.newBuilder().priorResponse(var4.newBuilder().body((ResponseBody)null).build()).build();
         }

         Request var51;
         if ((var51 = this.followUpRequest(var49)) == null) {
            if (!this.forWebSocket) {
               this.streamAllocation.release();
            }

            return var49;
         }

         Util.closeQuietly(var49.body());
         if (++var50 > 20) {
            this.streamAllocation.release();
            throw new ProtocolException("Too many follow-up requests: " + var50);
         }

         if (var51.body() instanceof UnrepeatableRequestBody) {
            throw new HttpRetryException("Cannot retry streamed HTTP body", var49.code());
         }

         if (!this.sameConnection(var49, var51.url())) {
            this.streamAllocation.release();
            ConnectionPool var53 = this.client.connectionPool();
            this.streamAllocation = new StreamAllocation(var53, this.createAddress(var51.url()));
         } else if (this.streamAllocation.stream() != null) {
            throw new IllegalStateException("Closing the body of " + var49 + " didn't close its backing stream. Bad interceptor?");
         }

         var4 = var49;
         var2 = var51;
      }

      this.streamAllocation.release();
      throw new IOException("Canceled");
   }
}
